﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Console_App
{
    class Serial
    {
        static byte[] lineBuffer = new byte[4096];
        static int lineBuf_writeCount = 0; // pozice
        static int lineBuf_readCount = 0; // pozice

        static SerialPort Port;

        public void PortInit()
        {

            try
            {
                Port = new SerialPort();
                Port.PortName = "COM19";
                Port.BaudRate = 5000000;              
                Port.Open();
            }
            catch
            {
                Console.WriteLine("Port Error!!!");
                return;
            }
            
            Console.WriteLine("Port: {0}", Port.IsOpen);
        }

        public void Read()
        {
            while (true)
            {
                ReadFromSerial();
            }
        }

        public void ReadFromSerial()
        {

            if (!Port.IsOpen) return;

            if (Port.BytesToRead > 0)
            {
                byte b = (byte)Port.ReadByte();
                lineBuffer[lineBuf_writeCount] = b;
                lineBuf_writeCount++;
                if (lineBuf_writeCount > 4096 - 1) lineBuf_writeCount = 0;
            }

            
        }

        public bool NewInBuffer()
        {
            return !(lineBuf_readCount == lineBuf_writeCount);
        }

        public byte ReadFromBuffer()
        {
            byte b = lineBuffer[lineBuf_readCount];
            lineBuf_readCount++;
            if (lineBuf_readCount > 4096 - 1) lineBuf_readCount = 0;

            return b;
        }
    }



    class Program
    {

        static void SaveData(StreamWriter sw, UInt16 data)
        {
            byte id = (byte)(data >> 11);

            UInt16 value = (UInt16)(data & 1023); //first 10 bites is value data 
            int OverFlow = ((data & (1 << 10)) >> 10);

            sw.Write(id);
            sw.Write(";");
            sw.Write(OverFlow);
            sw.Write(";");
            sw.Write(value);
            sw.WriteLine("");

        }


        //static  sw;


        
        static void Main(string[] args)
        {
            Console.WriteLine("App Start");
            Serial port = new Serial();

            string filename = DateTime.Now.Ticks.ToString() + ".csv";

            StreamWriter sw;

            try
            {
                sw = new StreamWriter(@filename);
                Console.WriteLine("Filename: " + filename);
            }
            catch
            {
                Console.WriteLine("File Error");
                return;
            }


            port.PortInit();

            Thread vlakno1 = new Thread(port.Read);
            vlakno1.Start();

            bool nibble = false;
            UInt16 data = 0;

            long DataCount = 0;

            while (true)
            {

                if (port.NewInBuffer() == true)
                {
                    byte b = port.ReadFromBuffer();


                    if (nibble == true) 
                    {
                        data |= b;

                        SaveData(sw, data);
                    }
                    else
                    {
                        data =  (UInt16)((b & 255) << 8);
                    }

                    DataCount++;
                    nibble = !nibble;
                }

                Console.SetCursorPosition(0, 4);
                Console.WriteLine("Number of received bytes: {0}", DataCount);

                Console.Write("\nPress e for end app and save data: ");
                if (Console.KeyAvailable)
                {
                    
                    string s = Console.ReadLine();

                    if (s == "e" || s == "E")
                    {
                        sw.Flush();
                        sw.Close();
                        Environment.Exit(0);
                    }
                }

            }
        }
    }
}
